import 'package:circular_splash_transition/controller.dart';
import 'package:flutter/material.dart';

class CircularSplash extends StatelessWidget {
  CircularSplash({Key key, @required this.controller, @required this.child})
      : duration = controller.duration,
        color = controller.color,
        super(key: key);

  final Duration duration;
  final Color color;
  final CircularSplashController controller;
  final Widget child;

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[
        child,
        CircularSplashPushAndPop(controller, duration, color),
      ],
    );
  }
}

class CircularSplashPushAndPop extends StatefulWidget {
  final CircularSplashController controller;
  final Duration duration;
  final Color color;

  const CircularSplashPushAndPop(this.controller, this.duration, this.color,
      {Key key})
      : super(key: key);

  @override
  _CircularSplashPushAndPopState createState() =>
      _CircularSplashPushAndPopState(controller, duration, color);
}

class _CircularSplashPushAndPopState extends State<CircularSplashPushAndPop>
    with SingleTickerProviderStateMixin {
  final CircularSplashController _bloc;
  AnimationController _animationController;
  Animation<double> _animationScale;
  final Duration _duration;
  final Color _color;

  _CircularSplashPushAndPopState(this._bloc, this._duration, this._color);

  @override
  void initState() {
    super.initState();
    _animationController =
        AnimationController(vsync: this, duration: _duration);
    _animationScale =
        Tween<double>(begin: 0.0, end: 1.0).animate(_animationController);
  }

  @override
  void dispose() {
    _animationController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<bool>(
      stream: _bloc.pushAndPopOut,
      initialData: false,
      builder: (BuildContext context, AsyncSnapshot<bool> snapshot) {
        if (snapshot.data) {
          _animationController.forward();
        }
        if (!snapshot.data) {
          if (_animationController.value > 0) {
            _animationController.reverse();
          } else {
            _animationController.reset();
          }
        }
        return _pushAndPop();
      },
    );
  }

  Widget _pushAndPop() {
    return AnimatedBuilder(
        animation: _animationScale,
        builder: (context, child) {
          return Center(
            child: Transform.scale(
              scale: _animationScale.value * 2.5,
              child: Container(
                height: MediaQuery.of(context).size.height,
                width: MediaQuery.of(context).size.height,
                decoration:
                BoxDecoration(shape: BoxShape.circle, color: _color),
              ),
            ),
          );
        });
  }
}
