import 'dart:async';
import 'package:flutter/material.dart';

class CircularSplashController {
  final Duration duration;
  final Color color;

  CircularSplashController(
      {this.duration = const Duration(milliseconds: 300),
      this.color = Colors.white});

  void dispose() {
    _pushAndPopController?.close();
  }

  final StreamController<bool> _pushAndPopController = StreamController<bool>();

  Stream<bool> get pushAndPopOut => _pushAndPopController.stream;

  Sink<bool> get pushAndPopIn => _pushAndPopController.sink;

  Future<Object> runPushNamed(BuildContext context, String routeName) async {
    pushAndPopIn.add(true);
    await Future.delayed(Duration(milliseconds: duration.inMilliseconds + 1));
    return await Navigator.pushNamed(context, routeName).then((value) {
      _runPop();
      return value;
    });
  }

  Future<Object> runPush(BuildContext context, Route route) async {
    pushAndPopIn.add(true);
    await Future.delayed(Duration(milliseconds: duration.inMilliseconds + 1));
    return await Navigator.push(context, route).then((value) {
      _runPop();
      return value;
    });
  }

  void runPushReplacementNamed(BuildContext context, String routeName) async {
    pushAndPopIn.add(true);
    await Future.delayed(Duration(milliseconds: duration.inMilliseconds + 1));
    Navigator.pushReplacementNamed(context, routeName);
    await Future.delayed(Duration(milliseconds: duration.inMilliseconds));
    dispose();
  }

  void _runPop() async {
    await Future.delayed(Duration(milliseconds: duration.inMilliseconds + 1));
    pushAndPopIn.add(false);
  }
}
